
#include "stdafx.h"

#include "ClassFactory.h"



CClassFactory::CClassFactory()
{
	
}

CClassFactory::~CClassFactory()
{
	//don't forget to clear the documents. Just in case
	clearClasDef();
	clearCurContext();
}

void CClassFactory::clearClasDef()
{
	//just clear the document
	classesDefinitions.Clear();
	
}

void CClassFactory::clearCurContext()
{
	//just clear the document
	currentContext.Clear();	
}

bool CClassFactory::loadContext(const char *filename)
{	
	//We load the file, and check Error
	if (! currentContext.LoadFile(filename)) 
	{								
		error("Error loading file = \"%s\" : %s.", filename,
			currentContext.ErrorDesc());		
		return false;		
	}
	else		
		return true;
	
}

bool CClassFactory::saveContext(const char *filename)
{	
	//We save the file, and check Error
	if (! currentContext.SaveFile(filename)) 
	{						
		error("Error saving file = \"%s\" : %s.", filename,
			currentContext.ErrorDesc());		
		return false;		
	}
	else		
		return true;	
}

bool CClassFactory::loadClassDef(const char *filename)
{		
	//We load the file, and check Error
	if (! classesDefinitions.LoadFile(filename)) 
	{		
		error("Error loading file = \"%s\" : %s.", filename,
			classesDefinitions.ErrorDesc());		
		return false;		
	}
	else		
		return true;		
}

bool CClassFactory::saveClassDef(const char *filename)
{
	//We save the file, and check Error
	if (! classesDefinitions.SaveFile(filename)) 
	{						
		error("Error saving file = \"%s\" : %s.", filename,
			classesDefinitions.ErrorDesc());		
		return false;		
	}
	else		
		return true;		
}

bool CClassFactory::newInstance(const char * className,
								const char* instanceName)
{
	TiXmlNode* classNode = findClass(className);
	
	//we din't find the class
	if (classNode == NULL)
	{		
		error("Couldn't find class %s",className);
		return false;		
	}
	else
	{		
		TiXmlElement* root = currentContext.RootElement();
		
		//if document it's empty
		if (root == NULL)
		{
			//create a root node
			currentContext.Parse ("<instances/>");
			root = currentContext.RootElement();
		}
		
		//new node for the instance
		TiXmlElement element("instance");
		
		//add the attributes to the instace
		if (addAttributesToInstance(&element, classNode))
		{			
			element.SetAttribute("class", className);
			element.SetAttribute("name", instanceName);

			//we add the instance it self
			root->InsertEndChild(element);			
		}
		else			
			return false;				
	}
	
	return true;
}

TiXmlNode* CClassFactory::findClass(const char *className)
{
	TiXmlNode* classNode = NULL;
	bool found = false;
	
	TiXmlElement* root = classesDefinitions.RootElement();
	
	//if the document is empty return NULL
	if (root == NULL)
	{		
		return NULL;
	}
	
	//first class def
	classNode = root->FirstChild("class");
	const char* classNodeName;
	
	//until be found it or they are no more class def
	while ((classNode != NULL) && (!found))
	{		
		//get the name of the class
		classNodeName = classNode->ToElement()->Attribute("name");
		
		//is the one?
		if (strcmpi(classNodeName , className) == 0)		
			found = true;			
		else		
			//go to the next
			classNode = classNode->NextSibling("class");
	}
			
	return ((found)?classNode:NULL);	
}

bool CClassFactory::deleteInstace(const char *instanceName)
{
	TiXmlNode* instanceNode = findInstance(instanceName);

	//if we donde find it the instance
	if (instanceNode == NULL)
	{		
		error("Couldn't find instance %s",instanceName);
		return false;		
	}
	else		
		//remove the instance node
		currentContext.RootElement()->RemoveChild(instanceNode);	
	
	return true;
}

TiXmlNode* CClassFactory::findInstance(const char *instanceName)
{
	TiXmlNode* instanceNode = NULL;
	bool found = false;
	
	TiXmlElement* root = currentContext.RootElement();

	//if the document is empty we return NULL
	if (root == NULL)
	{
		return NULL;
	}
	
	//get first instance node
	instanceNode = root->FirstChild("instance");
	const char* instanceNodeName;
	
	//until we found it or they are not more nodes
	while ((instanceNode != NULL) && (!found))
	{	
		//get instance name
		instanceNodeName = instanceNode->ToElement()->Attribute("name");
		
		//check if its the one
		if (strcmpi(instanceNodeName , instanceName) == 0)
			found = true;
		else
			//go to the next
			instanceNode = instanceNode->NextSibling("instance");
	}
	
	return (found)?instanceNode:NULL;	
}

bool CClassFactory::setValue(const char *instanceName, 
							 const char *attributeName, const char *value)
{
	TiXmlNode* instanceNode = findInstance(instanceName);
	
	//if we don't find the instance
	if (instanceNode == NULL)
	{
		error("Couldn't find instance %s",instanceName);
		return false;
	}
	else
	{
		TiXmlNode* attributeNode  = instanceNode->FirstChild(attributeName);

		//if we don't find the attribute
		if (attributeNode == NULL)
		{
			error("Couldn't find attribute %s",attributeName);
			return false;
		}
		else
		{
			//if the attribute have a value (values are childs from attribute)
			if (attributeNode->FirstChild() != NULL)
				//set the value
				attributeNode->FirstChild()->SetValue(value);
			else
			{
				//add the value as children
				TiXmlText text(value);
				attributeNode->InsertEndChild(text);
			}						
		}
	}
	return true;
}

CString CClassFactory::getValue(const char *instanceName,
								const char *attributeName)
{
	//by default "" value
	CString value;	
	value = "";
	
	TiXmlNode* instanceNode = findInstance(instanceName);
	
	//if we din't found the instance
	if (instanceNode == NULL)	
		error("Couldn't find instance %s",instanceName);		
	else
	{	
		TiXmlNode* attributeNode  = instanceNode->FirstChild(attributeName);
		
		//if we find the attribute
		if (attributeNode == NULL)
			error("Couldn't find attribute %s",attributeName);			
		else
			//if is have a value (values are childs from attribute)
			if (attributeNode->FirstChild() != NULL)
				value = attributeNode->FirstChild()->Value();			
	}
	return value;
}

bool CClassFactory::addAttributesToInstance(TiXmlElement *element,
											TiXmlNode *classNode)
{
	TiXmlNode* attributeNode;
	const char* attributeName;
	
	//we get the first attribute
	attributeNode = classNode->FirstChild("attribute");
	
	//until we have not more attributes
	while (attributeNode != NULL)
	{
		//get the name
		attributeName = attributeNode->ToElement()->Attribute("name");
		
		//add the attribute to the instance
		TiXmlElement attrInstance (attributeName);
		element->InsertEndChild(attrInstance);
		
		//next attribute node
		attributeNode = attributeNode->NextSibling("attribute");
	}
	
	const char* extends = NULL;
	
	extends = classNode->ToElement()->Attribute("extends");
	
	//if this class have a parent class
	if (extends != NULL)
	{
		TiXmlNode* parentClassNode = findClass(extends);

		//the parent class dosen't exist
		if (parentClassNode == NULL)
		{		
			error("Error couldn't find parent class %s",extends);
			return false;
		}
		else
			//add the attribute of the parent class (recursive)
			return addAttributesToInstance(element, parentClassNode);
	}
	
	return true;
}

bool CClassFactory::addClass(const char *className, const char *extends)
{
	TiXmlNode* classNode = findClass(className);
	
	//if we allready have that class
	if (classNode != NULL)
	{
		error("Class allready exits %s",className);
		return false;	
	}
	else
	{
		//new class def node
		TiXmlElement element("class");
		
		element.SetAttribute("name", className);
		
		//if we set a parent class
		if (extends != NULL)
		{
			//if we set a not empty class name
			if (strcmpi(extends,"") != 0)
			{
				classNode = findClass(extends);
				//if we din't find the parent class
				if (classNode == NULL)
				{
					error("Parent class dont found %s","extends");
					return false;	
				}
				else
					//set the parent as attribute of the node
					element.SetAttribute("extends", extends);
			}
		}
		
		TiXmlElement* root = classesDefinitions.RootElement();
		
		//if document its empty
		if (root == NULL)
		{	
			//add a root node
			classesDefinitions.Parse("<classes/>");
			root = classesDefinitions.RootElement();
		}
		
		//insert the class def
		root->InsertEndChild(element);
	}
	
	return true;;
}

bool CClassFactory::addAttribute(const char *className,
								 const char *attributeName)
{
	TiXmlNode* classNode = findClass(className);

	//if we din't find the class
	if (classNode == NULL)
	{
		error("Class dosen't exits %s",className);
		return false;	
	}
	else
	{
		const char* findAttributeName = NULL;
		
		bool found = false;

		//get the fist attribute node
		TiXmlNode* attributeNode = classNode->FirstChild("attribute");

		//until be found it or they aren't more attributes nodes
		while ((attributeNode != NULL) && (!found))
		{
			//get the attribute name
			findAttributeName = attributeNode->ToElement()->Attribute("name");
			
			//check if its the one
			if (strcmpi(findAttributeName , attributeName) == 0)			
				found = true;			
			else
				//go to the next attribute
				attributeNode = attributeNode->NextSibling("attribute");
		}

		//if we found it
		if (found)
		{
			error("Attribute allready defined for the class %s",attributeName);
			return false;	
		}
		else
		{
			//add the attribute to the class
			TiXmlElement element("attribute");
			
			element.SetAttribute("name", attributeName);
			
			classNode->ToElement()->InsertEndChild(element);
		}
	}
	
	return true;
}

bool CClassFactory::deleteClass(const char *className)
{
	TiXmlNode* classNode = findClass(className);
	
	//if we din't find the class
	if (classNode == NULL)
	{
		error("Cant find the class %s",className);
		return false;	
	}
	else
		//we remove the class def node. this olso remove all childrens
		classesDefinitions.RootElement()->RemoveChild(classNode);
	
	return true;
}

bool CClassFactory::deleteAttribute(const char *className,
									const char *attributeName)
{
	TiXmlNode* classNode = findClass(className);

	//if we din't find the class name
	if (classNode == NULL)
	{
		error("Class dosen't exits %s", className);
		return false;	
	}
	else
	{
		const char* findAttributeName = NULL;
		
		bool found = false;

		//get the first attribute node
		TiXmlNode* attributeNode = classNode->FirstChild("attribute");

		//until we found it or they aren't more attributes nodes
		while ((attributeNode != NULL) && (!found))
		{	
			//get the name
			findAttributeName = attributeNode->ToElement()->Attribute("name");
			
			//check if its the one
			if (strcmpi(findAttributeName , attributeName) == 0)			
				found = true;
			else
				//go to the next
				attributeNode = attributeNode->NextSibling("attribute");
		}
		//if we dont found it
		if (!found)
		{
			error("Attribute dosen't exits for this class %s",attributeName);
			return false;	
		}
		else
			//remove the attribute
			classNode->RemoveChild(attributeNode);
	}
	
	return true;
}

void CClassFactory::error(char message[], ...)
{
	//nasty code to display error messages

	static char messageBuffer[8192];   

	va_list argumentsPointer;
	
	va_start(argumentsPointer, message);	
	vsprintf(messageBuffer, message, argumentsPointer);
	va_end(argumentsPointer);
	
	MessageBox(NULL,messageBuffer,"Error", MB_OK | MB_ICONEXCLAMATION);
}
